package xyz.zhouxy.plusone.commons.util;

import java.util.function.Predicate;

/**
 * Predicates
 *
 * <p>
 * {@link Predicate} 相关操作。
 * </p>
 *
 * @author ZhouXY
 * @see Predicate
 */
public class Predicates {

    /**
     * 将 lambda 表达式或者方法引用包装为对应类型的 {@link Predicate} 对象。
     * 如将 {@code Objects::nonNull} 明确地指定为 {@code Predicate&lt;String&gt;}，
     * 使之可以链式调用 {@link Predicate#and(Predicate)}、{@link Predicate#or(Predicate)}
     * 等方法，连接其它 {@code Predicate<? super T>} 对象。
     * 
     * <pre>
     * Predicate&lt;String&gt; predicate = Predicates.&lt;String&gt;of(Objects::nonNull)
     *         .and(StringUtils::isNotEmpty);
     * </pre>
     * 
     * @param <T>       目标类型
     * @param predicate 原 {@link Predicate} 实例
     * @return 包装的 {@link Predicate} 实例
     */
    public static <T> Predicate<T> of(Predicate<? super T> predicate) {
        return predicate::test;
    }

    public static <T> Predicate<T> not(Predicate<? super T> predicate) {
        return t -> !predicate.test(t);
    }

    private Predicates() {
        throw new IllegalStateException("Utility class");
    }
}
