package xyz.zhouxy.plusone.validator;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.Supplier;

import xyz.zhouxy.plusone.validator.BaseValidator.*;

public abstract class MapValidator {
    private final List<Consumer<Map<String, Object>>> rules = new ArrayList<>();
    private final List<PropertyValidator<Map<String, Object>, ?, ?>> propertyValidators = new ArrayList<>();

    private final String[] propertyNames;

    protected MapValidator(List<String> propertyNames) {
        this.propertyNames = propertyNames.toArray(new String[0]);
    }

    protected void withRule(final Predicate<Map<String, Object>> rule, final String errorMessage) {
        withRule(rule, () -> new InvalidInputException(errorMessage));
    }

    protected <E extends RuntimeException> void withRule(Predicate<Map<String, Object>> rule,
            Supplier<E> exceptionBuilder) {
        withRule(rule, value -> exceptionBuilder.get());
    }

    protected <E extends RuntimeException> void withRule(Predicate<Map<String, Object>> condition,
            Function<Map<String, Object>, E> exceptionBuilder) {
        withRule(value -> {
            if (!condition.test(value)) {
                throw exceptionBuilder.apply(value);
            }
        });
    }

    protected void withRule(Consumer<Map<String, Object>> rule) {
        this.rules.add(rule);
    }

    @SuppressWarnings("unchecked")
    protected final <R> ObjectValidator<Map<String, Object>, R> ruleFor(String key) {
        ObjectValidator<Map<String, Object>, R> validValueHolder = new ObjectValidator<>(map -> (R) map.get(key));
        propertyValidators.add(validValueHolder);
        return validValueHolder;
    }

    protected final IntValidator<Map<String, Object>> ruleForInt(String key) {
        IntValidator<Map<String, Object>> validValueHolder = new IntValidator<>(map -> map.get(key));
        propertyValidators.add(validValueHolder);
        return validValueHolder;
    }

    protected final DoubleValidator<Map<String, Object>> ruleForDouble(String key) {
        DoubleValidator<Map<String, Object>> validValueHolder = new DoubleValidator<>(map -> map.get(key));
        propertyValidators.add(validValueHolder);
        return validValueHolder;
    }

    protected final BoolValidator<Map<String, Object>> ruleForBool(String key) {
        BoolValidator<Map<String, Object>> validValueHolder = new BoolValidator<>(map -> map.get(key));
        propertyValidators.add(validValueHolder);
        return validValueHolder;
    }

    protected final StringValidator<Map<String, Object>> ruleForString(String key) {
        StringValidator<Map<String, Object>> validValueHolder = new StringValidator<>(map -> map.get(key));
        propertyValidators.add(validValueHolder);
        return validValueHolder;
    }

    @SuppressWarnings("unchecked")
    protected final <E> CollectionValidator<Map<String, Object>, E> ruleForCollection(String key) {
        CollectionValidator<Map<String, Object>, E> validValueHolder = new CollectionValidator<>(
                m -> (Collection<E>) m.get(key));
        propertyValidators.add(validValueHolder);
        return validValueHolder;
    }

    public Map<String, Object> validate(Map<String, Object> obj) {
        return this.validate(obj, this.propertyNames);
    }

    public Map<String, Object> validate(Map<String, Object> obj, String... keys) {
        for (Consumer<Map<String, Object>> rule : this.rules) {
            rule.accept(obj);
        }
        for (PropertyValidator<Map<String, Object>, ?, ?> valueValidator : this.propertyValidators) {
            valueValidator.validate(obj);
        }
        final Map<String, Object> result = new HashMap<>(keys.length);
        for (String key : keys) {
            result.put(key, obj.get(key));
        }
        return result;
    }
}
